<?php

namespace Drupal\sendinblue\Form;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Menu\MenuLinkManagerInterface;
use Drupal\sendinblue\SendinblueMailin;
use Drupal\sendinblue\SendinblueManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Parameter Form to login SendinBlue.
 */
class ConfigurationSendinblueForm extends ConfigFormBase {
  /**
   * Sendinblue main class.
   *
   * @var \Drupal\sendinblue\SendinblueMailin
   */
  private $sendinblueMailin;
  /**
   * SendinblueManager.
   *
   * @var \Drupal\sendinblue\SendinblueManager
   */
  private $sendinblueManager;
  /**
   * CacheBackendInterface.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  private $cacheBackend;
  /**
   * MenuLinkManagerInterface.
   *
   * @var \Drupal\Core\Menu\MenuLinkManagerInterface
   */
  private $menuLinkManager;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\sendinblue\SendinblueMailin $sendinblueMailin
   *   Sendinblue main class.
   * @param \Drupal\sendinblue\SendinblueManager $sendinblueManager
   *   SendinblueManager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheBackend
   *   CacheBackendInterface.
   * @param \Drupal\Core\Menu\MenuLinkManagerInterface $menuLinkManager
   *   MenuLinkManagerInterface.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    SendinblueMailin $sendinblueMailin,
    SendinblueManager $sendinblueManager,
    CacheBackendInterface $cacheBackend,
    MenuLinkManagerInterface $menuLinkManager
  ) {
    parent::__construct($config_factory);
    $this->sendinblueMailin = $sendinblueMailin;
    $this->sendinblueManager = $sendinblueManager;
    $this->cacheBackend = $cacheBackend;
    $this->menuLinkManager = $menuLinkManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('sendinblue.mailing_service'),
      $container->get('sendinblue.manager'),
      $container->get('cache.menu'),
      $container->get('plugin.manager.menu.link')
    );
  }

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'sendinblue_form_login';
  }

  /**
   * Form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['access_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#required' => TRUE,
      '#attributes' => [
        'placeholder' => $this->t('API Key'),
      ],
      '#size' => 50,
      '#maxlenght' => 100,
    ];
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Login'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $access_key = $form_state->getValue('access_key');

    $this->sendinblueMailin->setApiKey($access_key);
    $response = $this->sendinblueMailin->getAccount();
    if (!is_array($response) || ($response['code'] !== 'success')) {
      if (!empty($access_key)) {
        $form_state->setErrorByName('access_key', $this->t('API key is invalid'));
      }
    }
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $access_key = $form_state->getValue('access_key');
    $config = $this->configFactory->getEditable(SendinblueManager::CONFIG_SETTINGS);

    $this->sendinblueMailin->setApiKey($access_key);
    $response = $this->sendinblueMailin->getAccount();

    if (is_array($response) && ($response['code'] === 'success')) {
      $account_data = $response['data'];
      $count = count($account_data);
      $account_email = $account_data[$count - 1]['email'];
      $account_user_name = $account_data[$count - 1]['first_name'] . ' ' . $account_data[$count - 1]['last_name'];
      $config->set(SendinblueManager::ACCESS_KEY, $access_key)->save();
      $config->set(SendinblueManager::ACCOUNT_EMAIL, $account_email)->save();
      $config->set(SendinblueManager::ACCOUNT_USERNAME, $account_user_name)
        ->save();
      $config->set(SendinblueManager::ACCOUNT_DATA, $account_data)->save();
      $smtp_details = $this->sendinblueManager->updateSmtpDetails();

      if (($smtp_details == FALSE) || ($smtp_details['relay'] == FALSE)) {
        $config->set('sendinblue_on', 0)->save();
      }
      else {
        $config->set('sendinblue_on', 1)->save();
      }

      $this->sendinblueMailin->partnerDrupal();
    }

    // Clear cache for menu tasks.
    $this->cacheBackend->invalidateAll();
    $this->menuLinkManager->rebuild();

    parent::submitForm($form, $form_state);
  }

  /**
   * Gets the configuration names that will be editable.
   *
   * @return array
   *   An array of configuration object names that are editable if called in
   *   conjunction with the trait's config() method.
   */
  protected function getEditableConfigNames() {
    return ['sendinblue_form_login.settings'];
  }

}
