<?php

namespace Drupal\sendinblue;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\sendinblue\Form\ConfigurationSendinblueForm;
use Drupal\sendinblue\Form\LogoutForm;
use Drupal\sendinblue\Form\RegisteringUserForm;
use Drupal\sendinblue\Form\TransactionnalEmailForm;

/**
 * Basic manager of module.
 */
class SendinblueManager {

  use StringTranslationTrait;

  const SENDINBLUE_SIGNUP_ENTITY = 'sendinblue_signup_form';
  const SENDINBLUE_SIGNUP_BLOCK = 1;
  const SENDINBLUE_SIGNUP_PAGE = 2;
  const SENDINBLUE_SIGNUP_BOTH = 3;

  /**
   * Variable name of Sendinblue URL.
   */
  const SIB_URL = 'https://my.sendinblue.com';

  /**
   * Variable name of Sendinblue access key.
   */
  const CONFIG_SETTINGS = 'sendinblue.config_global.settings';

  /**
   * Variable name of Sendinblue access key.
   */
  const CONFIG_SETTINGS_REGISTERING_USER = 'sendinblue.config_registering_user.settings';

  /**
   * Variable name of Sendinblue access key.
   */
  const CONFIG_SETTINGS_SEND_EMAIL = 'sendinblue.config_send_email.settings';

  /**
   * Variable name of Sendinblue access key.
   */
  const ACCESS_KEY = 'sendinblue_access_key';

  /**
   * Variable name of Sendinblue account email.
   */
  const ACCOUNT_EMAIL = 'sendinblue_account_email';

  /**
   * Variable name of Sendinblue account user name.
   */
  const ACCOUNT_USERNAME = 'sendinblue_account_username';

  /**
   * Variable name of Sendinblue account data.
   */
  const ACCOUNT_DATA = 'sendinblue_account_data';

  /**
   * Variable name of access_token.
   */
  const ACCESS_TOKEN = 'sendinblue_access_token';

  /**
   * Variable name of attribute lists.
   */
  const ATTRIBUTE_LISTS = 'sendinblue_attribute_lists';

  /**
   * Variable name of smtp details.
   */
  const SMTP_DETAILS = 'sendinblue_smtp_details';

  /**
   * ConfigFactoryInterface.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private $configFactory;
  /**
   * FormBuilderInterface.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  private $formBuilder;
  /**
   * Renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  private $renderer;
  /**
   * SendinblueMailin.
   *
   * @var SendinblueMailin
   */
  private $sendinblueMailin;
  /**
   * Connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  private $connection;
  /**
   * MailManagerInterface.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  private $mailManager;
  /**
   * AccountProxyInterface.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  private $accountProxy;

  /**
   * EntityModerationForm constructor.
   *
   * @param SendinblueMailin $sendinblueMailin
   *   SendinblueMailin.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   ConfigFactoryInterface.
   * @param \Drupal\Core\Database\Connection $connection
   *   Connection.
   * @param \Drupal\Core\Form\FormBuilderInterface $formBuilder
   *   FormBuilderInterface.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   Renderer.
   * @param \Drupal\Core\Mail\MailManagerInterface $mailManager
   *   MailManagerInterface.
   * @param \Drupal\Core\Session\AccountProxyInterface $accountProxy
   *   AccountProxyInterface.
   */
  public function __construct(
    SendinblueMailin $sendinblueMailin,
    ConfigFactoryInterface $configFactory,
    Connection $connection,
    FormBuilderInterface $formBuilder,
    Renderer $renderer,
    MailManagerInterface $mailManager,
    AccountProxyInterface $accountProxy
  ) {
    $this->connection = $connection;
    $this->configFactory = $configFactory;
    $this->formBuilder = $formBuilder;
    $this->renderer = $renderer;
    $this->sendinblueMailin = $sendinblueMailin;
    $this->mailManager = $mailManager;
    $this->accountProxy = $accountProxy;
  }

  /**
   * Get the access key store in configuration.
   *
   * @return string
   *   The SiB access key
   */
  public function getAccessKey() {
    return $this->configFactory->get(self::CONFIG_SETTINGS)->get(self::ACCESS_KEY);
  }

  /**
   * Get the account email store in configuration.
   *
   * @return string
   *   The SiB account email
   */
  public function getAccountEmail() {
    return $this->configFactory->get(self::CONFIG_SETTINGS)->get(self::ACCOUNT_EMAIL);
  }

  /**
   * Get the account username store in configuration.
   *
   * @return string
   *   The SiB account username
   */
  public function getAccountUsername() {
    return $this->configFactory->get(self::CONFIG_SETTINGS)->get(self::ACCOUNT_USERNAME);
  }

  /**
   * Get the data account store in configuration.
   *
   * @return string
   *   The SiB account store
   */
  public function getAccountData() {
    return $this->configFactory->get(self::CONFIG_SETTINGS)->get(self::ACCOUNT_DATA);
  }

  /**
   * Get the access token store in configuration.
   *
   * @return string
   *   The SiB access token
   */
  public function getAccessKeyToken() {
    return $this->configFactory->get(self::CONFIG_SETTINGS)->get(self::ACCESS_TOKEN);
  }

  /**
   * Generate Home layout of Log out.
   *
   * @return array
   *   A html of home page when log out.
   */
  public function generateHomeLogout() {
    $form = $this->formBuilder->getForm(ConfigurationSendinblueForm::class);

    return ['#formulaire_api_key' => $this->renderer->render($form)];
  }

  /**
   * Generate Home layout of Log out.
   *
   * @return array
   *   A html of home page when login.
   */
  public function generateHomeLogin() {

    // Calculate total count of subscribers.
    $list_response = $this->sendinblueMailin->getLists();
    if ($list_response['code'] !== 'success') {
      $total_subscribers = 0;
    }
    else {
      $list_datas = $list_response['data'];
      $list_ids = [];
      foreach ($list_datas as $list_data) {
        $list_ids[] = $list_data['id'];
      }
      $user_response = $this->sendinblueMailin->displayListUsers($list_ids, 1, 500);
      $total_subscribers = (int) $user_response['data']['total_list_records'];
    }

    // Get account details.
    $account_email = $this->getAccountEmail();
    $account_username = $this->getAccountUsername();
    $account_data = $this->getAccountData();

    $sendinblue_logout_form = $this->formBuilder->getForm(LogoutForm::class);
    $sendinblue_send_email_form = $this->formBuilder->getForm(TransactionnalEmailForm::class);
    $sendinblue_user_register_form = $this->formBuilder->getForm(RegisteringUserForm::class);

    return [
      '#account_username' => [
        '#plain_text' => $account_username,
      ],
      '#account_email' => [
        '#plain_text' => $account_email,
      ],
      '#total_subscribers' => [
        '#plain_text' => $total_subscribers,
      ],
      '#account_data' => $account_data,
      '#sendinblue_logout_form' => $this->renderer->render($sendinblue_logout_form),
      '#sendinblue_send_email_form' => $this->renderer->render($sendinblue_send_email_form),
      '#sendinblue_user_register_form' => $this->renderer->render($sendinblue_user_register_form),
    ];

  }

  /**
   * Generate List page when log in.
   *
   * @return string
   *   A html of list page.
   */
  public function generateListLogin() {
    $access_token = $this->updateAccessToken();
    return sprintf(self::SIB_URL . '/lists/index/access_token/%s', $access_token);
  }

  /**
   * Generate Campaign page when log in.
   *
   * @return string
   *   A html of campaign.
   */
  public function generateCampaignLogin() {
    $access_token = $this->updateAccessToken();
    return sprintf(self::SIB_URL . '/camp/listing/access_token/%s', $access_token);

  }

  /**
   * Generate Statistic page when log in.
   *
   * @return string
   *   A html of statistic page.
   */
  public function generateStatisticLogin() {
    $access_token = $this->updateAccessToken();
    return sprintf(self::SIB_URL . '/camp/message/access_token/%s', $access_token);
  }

  /**
   * Check if current state is logged in.
   *
   * @return bool
   *   A status of login of user.
   */
  public function isLoggedInState() {
    $access_key = $this->getAccessKey();
    if ($access_key != '') {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Update access token.
   *
   * @return string
   *   An access token information.
   */
  public function updateAccessToken() {
    $config = $this->configFactory->getEditable('sendinblue.settings');

    // If exist old access_token, delete it.
    $old_access_token = $config->get(self::ACCESS_TOKEN);
    $this->sendinblueMailin->deleteToken($old_access_token);

    // Get new access_token.
    $access_response = $this->sendinblueMailin->getAccessTokens();
    $access_token = $access_response['data']['access_token'];
    $config->set(self::ACCESS_TOKEN, $access_token);
    return $access_token;
  }

  /**
   * Get email template by type.
   *
   * @param string $type
   *   A type of email.
   *
   * @return array
   *   An array of email content.
   */
  public function getEmailTemplate($type = 'test') {
    $file = 'temp';
    $file_path = drupal_get_path('module', 'sendinblue') . '/asset/email-templates/' . $type . '/';
    // Get html content.
    $html_content = file_get_contents($file_path . $file . '.html');
    // Get text content.
    $text_content = file_get_contents($file_path . $file . '.txt');
    $templates = [
      'html_content' => $html_content,
      'text_content' => $text_content,
    ];
    return $templates;
  }

  /**
   * Send mail.
   *
   * @param string $type
   *   A type of email.
   * @param string $to_email
   *   A recipe address.
   * @param string $code
   *   A activate code.
   * @param int $list_id
   *   A list identification.
   * @param string $sender_id
   *   A sender identification.
   * @param string $template_id
   *   A template identification.
   */
  public function sendEmail($type, $to_email, $code, $list_id, $sender_id = '-1', $template_id = '-1') {
    $account_email = $this->getAccountEmail();
    $account_username = $this->getAccountUsername();
    $subject = '[SendinBlue]';

    // Set subject info.
    if ($type === 'confirm') {
      $subject = $this->t('Subscription confirmed');
    }
    elseif ($type === "double-optin") {
      $subject = $this->t('Please confirm subscription');
    }
    elseif ($type === 'test') {
      $subject = $this->t('[SendinBlue SMTP] test email');
    }

    $sender_email = $account_email;
    $sender_name = $account_username;

    if (empty($sender_email)) {
      $sender_email = $this->t('no-reply@sendinblue.com');
      $sender_name = $this->t('SendinBlue');
    }

    // Get template html and text.
    $template_contents = $this->getEmailTemplate($type);
    $html_content = $template_contents['html_content'];
    $text_content = $template_contents['text_content'];

    if ($type === "confirm" && $template_id !== '-1') {
      $response = $this->sendinblueMailin->getCampaign($template_id);
      if ($response['code'] === 'success') {
        $html_content = $response['data'][0]['html_content'];
        $subject = $response['data'][0]['subject'];
        if (($response['data'][0]['from_name'] !== '[DEFAULT_FROM_NAME]') &&
          ($response['data'][0]['from_email'] !== '[DEFAULT_FROM_EMAIL]') &&
          ($response['data'][0]['from_email'] !== '')
        ) {
          $sender_name = $response['data'][0]['from_name'];
          $sender_email = $response['data'][0]['from_email'];
        }
      }
    }

    // Send mail.
    $to = [$to_email => ''];
    $from = [$sender_email, $sender_name];
    $null_array = [];
    $base_url = $this->getBaseUrl();
    $site_domain = str_replace('https://', '', $base_url);
    $site_domain = str_replace('http://', '', $site_domain);

    $html_content = str_replace('{title}', $subject, $html_content);
    $html_content = str_replace('{site_domain}', $site_domain, $html_content);

    $text_content = str_replace('{site_domain}', $base_url, $text_content);
    $activate_email = $this->configFactory->get(self::CONFIG_SETTINGS_SEND_EMAIL)->get('sendinblue_on');

    if ($activate_email == '1') {
      $headers = [];
      $this->sendinblueMailin->sendEmail($to, $subject, $from, $html_content, $text_content, $null_array, $null_array, $from, $null_array, $headers);
    }
    else {
      $langcode = $this->accountProxy->getPreferredLangcode();

      $params['message'] = $html_content;
      $params['subject'] = $subject;
      $this->mailManager->mail('system', 'sendinblue_' . $type, $to_email, $langcode, $params, $sender_email);
    }
  }

  /**
   * Get Base URL.
   *
   * @return string
   *   A base url of the site.
   */
  public function getBaseUrl() {
    global $base_url;
    return $base_url;
  }

  /**
   * Get Attribute lists.
   *
   * @return array
   *   An array of attributes.
   */
  public function getAttributeLists() {
    $response = $this->sendinblueMailin->getAttributes();
    $config = $this->configFactory->getEditable('sendinblue.settings');

    if (($response['code'] === 'success') && (is_array($response['data']))) {
      $attributes = array_merge($response['data']['normal_attributes'], $response['data']['category_attributes']);
      $config->set(self::ATTRIBUTE_LISTS, $attributes);
      return $attributes;
    }
    return [];
  }

  /**
   * Get template list.
   *
   * @return array
   *   An array of template.
   */
  public function getTemplateList() {
    $response = $this->sendinblueMailin->getCampaigns('template');
    $templates = [
      [
        'id' => '-1',
        'name' => 'Default',
      ],
    ];
    if (($response['code'] === 'success') && (is_array($response['data']))) {
      foreach ($response['data']['campaign_records'] as $template) {
        $templates[] = [
          'id' => $template['id'],
          'name' => $template['campaign_name'],
        ];
      }
    }
    return $templates;
  }

  /**
   * Get lists.
   *
   * @return array
   *   An array of lists.
   */
  public function getLists() {
    $response = $this->sendinblueMailin->getLists();
    if (($response['code'] === 'success') && (is_array($response['data']))) {
      return $response['data'];
    }
    return [];
  }

  /**
   * Get list name form id.
   *
   * @param int $list_id
   *   A list id.
   *
   * @return string
   *   A list name.
   */
  public function getListNameById($list_id) {
    $response = $this->sendinblueMailin->getList($list_id);
    if (($response['code'] === 'success') && (is_array($response['data']))) {
      return $response['data']['name'];
    }
    return '';
  }

  /**
   * Get sender list.
   *
   * @return array
   *   An array of senders.
   */
  public function getSenderList() {
    $response = $this->sendinblueMailin->getSenders([]);
    $senders = [['id' => '-1', 'name' => 'Default']];
    if (($response['code'] === 'success') && (is_array($response['data']))) {
      foreach ($response['data'] as $sender) {
        $senders[] = [
          'id' => $sender['from_email'],
          'name' => $sender['from_email'],
        ];
      }
    }
    return $senders;
  }

  /**
   * Check the email address of subscriber.
   *
   * @param string $email
   *   An email address.
   * @param string $list_id
   *   A list id.
   *
   * @return array
   *   A response information.
   */
  public function validationEmail($email, $list_id) {
    $response = $this->sendinblueMailin->getUser($email);
    if ($response['code'] === 'failure') {
      $ret = [
        'code' => 'success',
        'listid' => [],
      ];
      return $ret;
    }

    $listid = $response['data']['listid'];
    if (!is_array($listid)) {
      $listid = [];
    }
    if ($response['data']['blacklisted'] == 1) {
      $ret = [
        'code' => 'update',
        'listid' => $listid,
      ];
    }
    else {
      if (!in_array($list_id, $listid)) {
        $ret = [
          'code' => 'success',
          'listid' => $listid,
        ];
      }
      else {
        $ret = [
          'code' => 'already_exist',
          'listid' => $listid,
        ];
      }
    }
    return $ret;
  }

  /**
   * Subscriber user.
   *
   * @param string $email
   *   An email address of subscriber.
   * @param array $info
   *   A data of subscriber.
   * @param array $listids
   *   An array of list id.
   *
   * @return string
   *   A response information.
   */
  public function subscribeUser($email, array $info = [], array $listids = []) {
    $response = $this->sendinblueMailin->createUpdateUser($email, $info, [], $listids, NULL);
    return $response['code'];
  }

  /**
   * Get subscriber data by email on drupal table.
   *
   * @param string $email
   *   An email address.
   *
   * @return string
   *   A details of subscriber.
   */
  public function getSubscriberByEmail($email) {
    $record = $this->connection->select('sendinblue_contact', 'c')
      ->fields('c', ['email'])
      ->condition('c.email', $email)
      ->execute()->fetchAssoc();

    return $record;
  }

  /**
   * Add subscriber on drupal table.
   *
   * @param array $data
   *   A data to add in table.
   */
  public function addSubscriberTable(array $data = []) {
    $this->connection->insert('sendinblue_contact')->fields(
      [
        'email' => $data['email'],
        'info' => $data['info'],
        'code' => $data['code'],
        'is_active' => $data['is_active'],
      ]
    )->execute();
  }

  /**
   * Update smtp details.
   *
   * @return string|bool
   *   A access token if exist, else 0.
   */
  public function updateSmtpDetails() {
    $response = $this->sendinblueMailin->getSmtpDetails();
    $config = $this->configFactory->getEditable(self::CONFIG_SETTINGS_SEND_EMAIL);

    if ($response['code'] === 'success') {
      if ($response['data']['relay_data']['status'] === 'enabled') {
        $smtp_details = $response['data']['relay_data']['data'];
        $config->set(self::SMTP_DETAILS, $smtp_details)->save();
        return $smtp_details;
      }
      else {
        $smtp_details = [
          'relay' => FALSE,
        ];
        $config->set('sendinblue_on', 0)->save();
        $config->set(self::SMTP_DETAILS, $smtp_details)->save();
        return $smtp_details;
      }
    }

    return FALSE;
  }

}
